#ifndef STRINGBUFFER_H
#define STRINGBUFFER_H

#include <cstring>

template<int N>
class StringBuffer final {
    int length;
    char data[N];

public:

    StringBuffer() : length(0) {
        data[0] = '\0';
    }

    StringBuffer(const char* str) : StringBuffer() {
        append(str);
    }

    bool operator==(const char* str) const {
        return strcmp(data, str) == 0;
    }

    bool operator==(const StringBuffer& other) const {
        return length == other.length && other == data;
    }

    bool operator!=(const char* str) const {
        return !((*this) == str);
    }

    bool operator!=(const StringBuffer& other) const {
        return !((*this) == other);
    }

    operator const char*() const {
        return data;
    }

    char operator[](int index) const {
        return data[index];
    }

    int getLength() const {
        return length;
    }

    //StringBuffer& append(char c);

    StringBuffer& append(const char* str) {
        for(int i = 0; length < N - 1 && str[i] != '\0'; length++, i++) {
            data[length] = str[i];
        }
        data[length] = '\0';
        return *this;
    }

    template<typename T>
    StringBuffer& append(const char* format, const T& t) {
        int left = N - length;
        int written = snprintf(data + length, left, format, t);
        if(written < left) {
            length += written;
        } else {
            length = N - 1;
        }
        return *this;
    }

    StringBuffer& append(int i) {
        return append("%d", i);
    }

    StringBuffer& append(float i) {
        return append("%.2f", i);
    }

    StringBuffer& append(bool b) {
        return b ? append("true") : append("false");
    }

    StringBuffer& clear() {
        length = 0;
        data[0] = '\0';
        return *this;
    }
};

template<int N>
bool operator==(const char* str, const StringBuffer<N> buffer) {
    return buffer == str;
}

template<int N>
bool operator!=(const char* str, const StringBuffer<N> buffer) {
    return buffer != str;
}

#endif